@@ -1,9 +1,6 @@
 /***************************************************************************
- $RCSfile$
-                             -------------------
-    cvs         : $Id: card.c 187 2006-06-15 16:13:23Z martin $
     begin       : Mon Mar 01 2004
-    copyright   : (C) 2004 by Martin Preuss
+    copyright   : (C) 2004-2010 by Martin Preuss
     email       : martin@libchipcard.de
 
  ***************************************************************************
@@ -27,6 +24,77 @@
 #include <gwenhywfar/gwentime.h>
 #include <chipcard/chipcard.h>
 
+#ifdef OS_WIN32
+# define CM_IOCTL_GET_FEATURE_REQUEST SCARD_CTL_CODE(3400)
+
+# define FEATURE_VERIFY_PIN_START  0x01 /* OMNIKEY Proposal */
+# define FEATURE_VERIFY_PIN_FINISH 0x02 /* OMNIKEY Proposal */
+# define FEATURE_MODIFY_PIN_START  0x03 /* OMNIKEY Proposal */
+# define FEATURE_MODIFY_PIN_FINISH 0x04 /* OMNIKEY Proposal */
+# define FEATURE_GET_KEY_PRESSED   0x05 /* OMNIKEY Proposal */
+# define FEATURE_VERIFY_PIN_DIRECT 0x06 /* USB CCID PIN Verify */
+# define FEATURE_MODIFY_PIN_DIRECT 0x07 /* USB CCID PIN Modify */
+# define FEATURE_MCT_READERDIRECT  0x08 /* KOBIL Proposal */
+# define FEATURE_MCT_UNIVERSAL     0x09 /* KOBIL Proposal */
+# define FEATURE_IFD_PIN_PROP      0x0A /* Gemplus Proposal */
+# define FEATURE_ABORT             0x0B /* SCM Proposal */
+
+/* Set structure elements aligment on bytes
+ * http://gcc.gnu.org/onlinedocs/gcc/Structure_002dPacking-Pragmas.html */
+#pragma pack(push, 1)
+
+/* the structure must be 6-bytes long */
+typedef struct {
+  uint8_t tag;
+  uint8_t length;
+  uint32_t value;
+} PCSC_TLV_STRUCTURE;
+
+#pragma pack(pop)
+
+
+#elif defined (OS_DARWIN)
+# define SCARD_CTL_CODE(code) (0x42000000 + (code))
+# define CM_IOCTL_GET_FEATURE_REQUEST SCARD_CTL_CODE(3400)
+
+# define FEATURE_VERIFY_PIN_START  0x01 /* OMNIKEY Proposal */
+# define FEATURE_VERIFY_PIN_FINISH 0x02 /* OMNIKEY Proposal */
+# define FEATURE_MODIFY_PIN_START  0x03 /* OMNIKEY Proposal */
+# define FEATURE_MODIFY_PIN_FINISH 0x04 /* OMNIKEY Proposal */
+# define FEATURE_GET_KEY_PRESSED   0x05 /* OMNIKEY Proposal */
+# define FEATURE_VERIFY_PIN_DIRECT 0x06 /* USB CCID PIN Verify */
+# define FEATURE_MODIFY_PIN_DIRECT 0x07 /* USB CCID PIN Modify */
+# define FEATURE_MCT_READERDIRECT  0x08 /* KOBIL Proposal */
+# define FEATURE_MCT_UNIVERSAL     0x09 /* KOBIL Proposal */
+# define FEATURE_IFD_PIN_PROP      0x0A /* Gemplus Proposal */
+# define FEATURE_ABORT             0x0B /* SCM Proposal */
+
+/* Set structure elements aligment on bytes
+ * http://gcc.gnu.org/onlinedocs/gcc/Structure_002dPacking-Pragmas.html */
+#ifdef __APPLE__
+#pragma pack(1)
+#else
+#pragma pack(push, 1)
+#endif
+
+/* the structure must be 6-bytes long */
+typedef struct {
+  uint8_t tag;
+  uint8_t length;
+  uint32_t value;
+} PCSC_TLV_STRUCTURE;
+
+#ifdef __APPLE__
+#pragma pack()
+#else
+#pragma pack(pop)
+#endif
+
+#else
+# include <PCSC/reader.h>
+#endif
+
+
 #include <stdlib.h>
 #include <assert.h>
 #include <string.h>
@@ -38,22 +106,23 @@ GWEN_LIST2_FUNCTIONS(LC_CARD, LC_Card)
 
 
 LC_CARD *LC_Card_new(LC_CLIENT *cl,
-                     uint32_t cardId,
+                     SCARDHANDLE scardHandle,
+                     const char *readerName,
+                     DWORD protocol,
                      const char *cardType,
                      uint32_t rflags,
                      const unsigned char *atrBuf,
-                     unsigned int atrLen) {
+                     unsigned int atrLen)
+{
   LC_CARD *cd;
 
   assert(cl);
-  assert(cardId);
   assert(cardType);
 
   GWEN_NEW_OBJECT(LC_CARD, cd);
   GWEN_LIST_INIT(LC_CARD, cd);
   GWEN_INHERIT_INIT(LC_CARD, cd);
   cd->client=cl;
-  cd->cardId=cardId;
   cd->cardType=strdup(cardType);
   cd->readerFlags=rflags;
   cd->cardTypes=GWEN_StringList_new();
@@ -61,7 +130,7 @@ LC_CARD *LC_Card_new(LC_CLIENT *cl,
   cd->usage=1;
   if (atrBuf && atrLen) {
     cd->atr=GWEN_Buffer_new(0, atrLen, 0, 1);
-    GWEN_Buffer_AppendBytes(cd->atr, (const char*)atrBuf, atrLen);
+    GWEN_Buffer_AppendBytes(cd->atr, (const char *)atrBuf, atrLen);
   }
 
   cd->openFn=LC_Card__Open;
@@ -82,12 +151,17 @@ LC_CARD *LC_Card_new(LC_CLIENT *cl,
     }
   }
 
+  cd->readerName=strdup(readerName);
+  cd->scardHandle=scardHandle;
+  cd->protocol=protocol;
+
   return cd;
 }
 
 
 
-void LC_Card_free(LC_CARD *cd){
+void LC_Card_free(LC_CARD *cd)
+{
   if (cd) {
     assert(cd->usage>0);
     cd->usage--;
@@ -97,6 +171,7 @@ void LC_Card_free(LC_CARD *cd){
         DBG_WARN(LC_LOGDOMAIN,
                  "Card to be deleted is still connected");
       }
+      free(cd->readerName);
       free(cd->cardType);
       free(cd->lastResult);
       free(cd->lastText);
@@ -111,7 +186,8 @@ void LC_Card_free(LC_CARD *cd){
 
 
 
-void LC_Card_List2_freeAll(LC_CARD_LIST2 *l){
+void LC_Card_List2_freeAll(LC_CARD_LIST2 *l)
+{
   if (l) {
     LC_CARD_LIST2_ITERATOR *cit;
 
@@ -120,7 +196,7 @@ void LC_Card_List2_freeAll(LC_CARD_LIST2 *l){
       LC_CARD *card;
 
       card=LC_Card_List2Iterator_Data(cit);
-      while(card) {
+      while (card) {
         LC_CARD *next;
 
         next=LC_Card_List2Iterator_Next(cit);
@@ -135,28 +211,40 @@ void LC_Card_List2_freeAll(LC_CARD_LIST2 *l){
 
 
 
-int LC_Card_IsConnected(const LC_CARD *card) {
+SCARDHANDLE LC_Card_GetSCardHandle(const LC_CARD *card)
+{
+  assert(card);
+  return card->scardHandle;
+}
+
+
+
+int LC_Card_IsConnected(const LC_CARD *card)
+{
   assert(card);
   return card->connected;
 }
 
 
 
-void LC_Card_SetConnected(LC_CARD *card, int b) {
+void LC_Card_SetConnected(LC_CARD *card, int b)
+{
   assert(card);
   card->connected=b;
 }
 
 
 
-const char *LC_Card_GetReaderType(const LC_CARD *cd) {
+const char *LC_Card_GetReaderType(const LC_CARD *cd)
+{
   assert(cd);
   return cd->readerType;
 }
 
 
 
-void LC_Card_SetReaderType(LC_CARD *cd, const char *s) {
+void LC_Card_SetReaderType(LC_CARD *cd, const char *s)
+{
   assert(cd);
   free(cd->readerType);
   if (s)
@@ -167,14 +255,16 @@ void LC_Card_SetReaderType(LC_CARD *cd, const char *s) {
 
 
 
-const char *LC_Card_GetDriverType(const LC_CARD *cd) {
+const char *LC_Card_GetDriverType(const LC_CARD *cd)
+{
   assert(cd);
   return cd->driverType;
 }
 
 
 
-void LC_Card_SetDriverType(LC_CARD *cd, const char *s) {
+void LC_Card_SetDriverType(LC_CARD *cd, const char *s)
+{
   assert(cd);
   free(cd->driverType);
   if (s)
@@ -185,14 +275,16 @@ void LC_Card_SetDriverType(LC_CARD *cd, const char *s) {
 
 
 
-const GWEN_STRINGLIST *LC_Card_GetCardTypes(const LC_CARD *cd){
+const GWEN_STRINGLIST *LC_Card_GetCardTypes(const LC_CARD *cd)
+{
   assert(cd);
   return cd->cardTypes;
 }
 
 
 
-int LC_Card_AddCardType(LC_CARD *cd, const char *s) {
+int LC_Card_AddCardType(LC_CARD *cd, const char *s)
+{
   assert(cd);
   return GWEN_StringList_AppendString(cd->cardTypes, s, 0, 1);
 }
@@ -202,7 +294,8 @@ int LC_Card_AddCardType(LC_CARD *cd, const char *s) {
 void LC_Card_SetLastResult(LC_CARD *cd,
                            const char *result,
                            const char *text,
-                           int sw1, int sw2){
+                           int sw1, int sw2)
+{
   assert(cd);
   free(cd->lastResult);
   free(cd->lastText);
@@ -219,63 +312,64 @@ void LC_Card_SetLastResult(LC_CARD *cd,
 }
 
 
-int LC_Card_GetLastSW1(const LC_CARD *cd){
+int LC_Card_GetLastSW1(const LC_CARD *cd)
+{
   assert(cd);
   return cd->lastSW1;
 }
 
 
 
-int LC_Card_GetLastSW2(const LC_CARD *cd){
+int LC_Card_GetLastSW2(const LC_CARD *cd)
+{
   assert(cd);
   return cd->lastSW2;
 }
 
 
 
-const char *LC_Card_GetLastResult(const LC_CARD *cd){
+const char *LC_Card_GetLastResult(const LC_CARD *cd)
+{
   assert(cd);
   return cd->lastResult;
 }
 
 
 
-const char *LC_Card_GetLastText(const LC_CARD *cd){
+const char *LC_Card_GetLastText(const LC_CARD *cd)
+{
   assert(cd);
   return cd->lastText;
 }
 
 
 
-LC_CLIENT *LC_Card_GetClient(const LC_CARD *cd){
+LC_CLIENT *LC_Card_GetClient(const LC_CARD *cd)
+{
   assert(cd);
   return cd->client;
 }
 
 
 
-uint32_t LC_Card_GetCardId(const LC_CARD *cd){
-  assert(cd);
-  return cd->cardId;
-}
-
-
-
-uint32_t LC_Card_GetReaderFlags(const LC_CARD *cd){
+uint32_t LC_Card_GetReaderFlags(const LC_CARD *cd)
+{
   assert(cd);
   return cd->readerFlags;
 }
 
 
 
-const char *LC_Card_GetCardType(const LC_CARD *cd){
+const char *LC_Card_GetCardType(const LC_CARD *cd)
+{
   assert(cd);
   return cd->cardType;
 }
 
 
 
-void LC_Card_SetCardType(LC_CARD *cd, const char *ct){
+void LC_Card_SetCardType(LC_CARD *cd, const char *ct)
+{
   assert(cd);
   assert(ct);
 
@@ -285,14 +379,15 @@ void LC_Card_SetCardType(LC_CARD *cd, const char *ct){
 
 
 
-unsigned int LC_Card_GetAtr(const LC_CARD *cd, const unsigned char **pbuf){
+unsigned int LC_Card_GetAtr(const LC_CARD *cd, const unsigned char **pbuf)
+{
   assert(cd);
   if (cd->atr) {
     unsigned int len;
 
     len=GWEN_Buffer_GetUsedBytes(cd->atr);
     if (len) {
-      *pbuf=(const unsigned char*)GWEN_Buffer_GetStart(cd->atr);
+      *pbuf=(const unsigned char *)GWEN_Buffer_GetStart(cd->atr);
       return len;
     }
   }
@@ -301,57 +396,73 @@ unsigned int LC_Card_GetAtr(const LC_CARD *cd, const unsigned char **pbuf){
 
 
 
-void LC_Card_ResetCardId(LC_CARD *cd){
+uint32_t LC_Card_GetFeatureCode(const LC_CARD *cd, int idx)
+{
   assert(cd);
-  cd->cardId=0;
+  assert(idx<LC_PCSC_MAX_FEATURES);
+  return cd->featureCode[idx];
 }
 
 
 
-void LC_Card_Dump(const LC_CARD *cd, FILE *f, int insert) {
+const char *LC_Card_GetReaderName(const LC_CARD *card)
+{
+  assert(card);
+  return card->readerName;
+}
+
+
+
+DWORD LC_Card_GetProtocol(const LC_CARD *card)
+{
+  assert(card);
+  return card->protocol;
+}
+
+
+
+void LC_Card_Dump(const LC_CARD *cd, int insert)
+{
   int k;
   GWEN_STRINGLISTENTRY *se;
   GWEN_DB_NODE *dbT;
 
   assert(cd);
   for (k=0; k<insert; k++)
-    fprintf(f, " ");
-  fprintf(f, "Card\n");
+    fprintf(stderr, " ");
+  fprintf(stderr, "Card\n");
   for (k=0; k<insert; k++)
-    fprintf(f, " ");
-  fprintf(f,
+    fprintf(stderr, " ");
+  fprintf(stderr,
           "==================="
           "==================="
           "==================="
           "==================\n");
   for (k=0; k<insert; k++)
-    fprintf(f, " ");
-  fprintf(f, "Card id       : %08x\n", cd->cardId);
+    fprintf(stderr, " ");
+  fprintf(stderr, "Card type     : %s\n", cd->cardType);
   for (k=0; k<insert; k++)
-    fprintf(f, " ");
-  fprintf(f, "Card type     : %s\n", cd->cardType);
+    fprintf(stderr, " ");
+  fprintf(stderr, "Driver type   : %s\n", cd->driverType);
   for (k=0; k<insert; k++)
-    fprintf(f, " ");
-  fprintf(f, "Driver type   : %s\n", cd->driverType);
+    fprintf(stderr, " ");
+  fprintf(stderr, "Reader type   : %s\n", cd->readerType);
   for (k=0; k<insert; k++)
-    fprintf(f, " ");
-  fprintf(f, "Reader type   : %s\n", cd->readerType);
-  for (k=0; k<insert; k++)
-    fprintf(f, " ");
-  fprintf(f, "Card types    :");
+    fprintf(stderr, " ");
+  fprintf(stderr, "Card types    :");
   se=GWEN_StringList_FirstEntry(cd->cardTypes);
-  while(se) {
+  while (se) {
     const char *s;
 
     s=GWEN_StringListEntry_Data(se);
     assert(s);
-    fprintf(f, " %s", s);
+    fprintf(stderr, " %s", s);
     se=GWEN_StringListEntry_Next(se);
   } /* while */
-  fprintf(f, "\n");
+  fprintf(stderr, "\n");
   for (k=0; k<insert; k++)
-    fprintf(f, " ");
-  fprintf(f, "Reader flags  : ");
+    fprintf(stderr, " ");
+  fprintf(stderr, "Reader flags  : ");
 
   dbT=GWEN_DB_Group_new("flags");
   LC_ReaderFlags_toDb(dbT, "flags", cd->readerFlags);
@@ -362,31 +473,30 @@ void LC_Card_Dump(const LC_CARD *cd, FILE *f, int insert) {
     if (!s)
       break;
     if (k)
-      fprintf(f, ", ");
-    fprintf(f, "%s", s);
+      fprintf(stderr, ", ");
+    fprintf(stderr, "%s", s);
   }
-  fprintf(f, "\n");
+  fprintf(stderr, "\n");
   GWEN_DB_Group_free(dbT);
 
   if (cd->atr) {
     for (k=0; k<insert; k++)
-      fprintf(f, " ");
-    fprintf(f, "ATR\n");
+      fprintf(stderr, " ");
+    fprintf(stderr, "ATR\n");
     for (k=0; k<insert; k++)
-      fprintf(f, " ");
-    fprintf(f,
+      fprintf(stderr, " ");
+    fprintf(stderr,
             "-------------------"
             "-------------------"
             "-------------------"
             "------------------\n");
     GWEN_Text_DumpString(GWEN_Buffer_GetStart(cd->atr),
                          GWEN_Buffer_GetUsedBytes(cd->atr),
-                         f,
                          insert+2);
   }
   for (k=0; k<insert; k++)
-    fprintf(f, " ");
-  fprintf(f,
+    fprintf(stderr, " ");
+  fprintf(stderr,
           "==================="
           "==================="
           "==================="
@@ -395,9 +505,73 @@ void LC_Card_Dump(const LC_CARD *cd, FILE *f, int insert) {
 
 
 
-LC_CLIENT_RESULT LC_Card_Open(LC_CARD *card) {
+LC_CLIENT_RESULT LC_Card_ReadFeatures(LC_CARD *card)
+{
+  LONG rv;
+  unsigned char rbuffer[300];
+  DWORD rblen;
+
   assert(card);
 
+  /* get control codes */
+  DBG_INFO(LC_LOGDOMAIN, "Reading control codes for CCID features");
+  rv=SCardControl(card->scardHandle,
+                  CM_IOCTL_GET_FEATURE_REQUEST,
+                  NULL,
+                  0,
+                  rbuffer,
+                  sizeof(rbuffer),
+                  &rblen);
+  if (rv!=SCARD_S_SUCCESS) {
+    DBG_INFO(LC_LOGDOMAIN,
+             "SCardControl: %04lx", (long unsigned int) rv);
+  }
+  else {
+    int cnt;
+    PCSC_TLV_STRUCTURE *tlv;
+    int i;
+
+    /* clear keypad flag; if there is TLV indicating the reader has a keypad and
+     * the driver supports it we set the flag upon encounter of the tlv */
+    card->readerFlags&=~LC_READER_FLAGS_KEYPAD;
+    cnt=rblen/sizeof(PCSC_TLV_STRUCTURE);
+    tlv=(PCSC_TLV_STRUCTURE *)rbuffer;
+    for (i=0; i<cnt; i++) {
+      uint32_t v;
+
+      v=tlv[i].value;
+#ifdef LC_ENDIAN_LITTLE
+      v=((v & 0xff000000)>>24) |
+        ((v & 0x00ff0000)>>8) |
+        ((v & 0x0000ff00)<<8) |
+        ((v & 0x000000ff)<<24);
+#endif
+      DBG_INFO(LC_LOGDOMAIN, "Feature %d: %08x", tlv[i].tag, v);
+      if (tlv[i].tag==FEATURE_VERIFY_PIN_DIRECT)
+        card->readerFlags|=LC_READER_FLAGS_KEYPAD;
+      if (tlv[i].tag<LC_PCSC_MAX_FEATURES) {
+        card->featureCode[tlv[i].tag]=v;
+      }
+    }
+  }
+
+  /* done */
+  return LC_Client_ResultOk;
+}
+
+
+
+LC_CLIENT_RESULT LC_Card_Open(LC_CARD *card)
+{
+  LONG rv;
+
+  assert(card);
+
+  rv=LC_Card_ReadFeatures(card);
+  if (rv!=LC_Client_ResultOk) {
+    DBG_INFO(LC_LOGDOMAIN, "here (%d)", (int) rv);
+  }
+
   LC_Card_SetLastResult(card, 0, 0, -1, -1);
   if (!card->openFn) {
     DBG_DEBUG(LC_LOGDOMAIN, "No OpenFn set");
@@ -408,7 +582,8 @@ LC_CLIENT_RESULT LC_Card_Open(LC_CARD *card) {
 
 
 
-LC_CLIENT_RESULT LC_Card_Close(LC_CARD *card) {
+LC_CLIENT_RESULT LC_Card_Close(LC_CARD *card)
+{
   LC_CLIENT_RESULT res;
 
   assert(card);
@@ -424,13 +599,15 @@ LC_CLIENT_RESULT LC_Card_Close(LC_CARD *card) {
 
 
 
-LC_CLIENT_RESULT CHIPCARD_CB LC_Card__Open(LC_CARD *card) {
+LC_CLIENT_RESULT CHIPCARD_CB LC_Card__Open(LC_CARD *card)
+{
   return LC_Client_ResultOk;
 }
 
 
 
-LC_CLIENT_RESULT CHIPCARD_CB LC_Card__Close(LC_CARD *card) {
+LC_CLIENT_RESULT CHIPCARD_CB LC_Card__Close(LC_CARD *card)
+{
   return LC_Client_ResultOk;
 }
 
@@ -440,8 +617,8 @@ LC_CLIENT_RESULT LC_Card_ExecApdu(LC_CARD *card,
                                   const char *apdu,
                                   unsigned int len,
                                   GWEN_BUFFER *rbuf,
-                                  LC_CLIENT_CMDTARGET t,
-                                  int timeout) {
+                                  LC_CLIENT_CMDTARGET t)
+{
   assert(card);
   assert(card->client);
   LC_Card_SetLastResult(card, 0, 0, -1, -1);
@@ -450,8 +627,7 @@ LC_CLIENT_RESULT LC_Card_ExecApdu(LC_CARD *card,
                             apdu,
                             len,
                             rbuf,
-                            t,
-                            timeout);
+                            t);
 }
 
 
@@ -459,8 +635,8 @@ LC_CLIENT_RESULT LC_Card_ExecApdu(LC_CARD *card,
 LC_CLIENT_RESULT LC_Card_ExecCommand(LC_CARD *card,
                                      const char *commandName,
                                      GWEN_DB_NODE *cmdData,
-                                     GWEN_DB_NODE *rspData,
-                                     int timeout) {
+                                     GWEN_DB_NODE *rspData)
+{
   LC_CLIENT_RESULT res;
 
   assert(card);
@@ -470,15 +646,15 @@ LC_CLIENT_RESULT LC_Card_ExecCommand(LC_CARD *card,
                             card,
                             commandName,
                             cmdData,
-                            rspData,
-                            timeout);
+                            rspData);
   return res;
 }
 
 
 
 GWEN_XMLNODE *LC_Card_FindCommand(LC_CARD *card,
-                                  const char *commandName) {
+                                  const char *commandName)
+{
   GWEN_DB_NODE *db;
   GWEN_XMLNODE *node;
 
@@ -495,13 +671,13 @@ GWEN_XMLNODE *LC_Card_FindCommand(LC_CARD *card,
     assert(db);
   }
 
-  node=(GWEN_XMLNODE*)GWEN_DB_GetPtrValue(db, commandName, 0, 0);
+  node=(GWEN_XMLNODE *)GWEN_DB_GetPtrValue(db, commandName, 0, 0);
   if (node==0) {
     node=LC_Client_FindCardCommand(card->client, card, commandName);
     if (node)
       GWEN_DB_SetPtrValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
                           commandName,
-                          (void*) node);
+                          (void *) node);
   }
   else {
     DBG_INFO(LC_LOGDOMAIN, "Found command \"%s\" in cache", commandName);
@@ -515,7 +691,8 @@ GWEN_XMLNODE *LC_Card_FindCommand(LC_CARD *card,
 LC_CLIENT_RESULT LC_Card_BuildApdu(LC_CARD *card,
                                    const char *command,
                                    GWEN_DB_NODE *cmdData,
-                                   GWEN_BUFFER *gbuf) {
+                                   GWEN_BUFFER *gbuf)
+{
   assert(card);
   assert(card->client);
   return LC_Client_BuildApdu(card->client,
@@ -528,7 +705,8 @@ LC_CLIENT_RESULT LC_Card_BuildApdu(LC_CARD *card,
 
 
 
-LC_CLIENT_RESULT LC_Card_SelectApp(LC_CARD *card, const char *appName) {
+LC_CLIENT_RESULT LC_Card_SelectApp(LC_CARD *card, const char *appName)
+{
   GWEN_XMLNODE *node;
 
   node=LC_Client_GetAppNode(card->client, appName);
@@ -544,14 +722,16 @@ LC_CLIENT_RESULT LC_Card_SelectApp(LC_CARD *card, const char *appName) {
 
 
 
-GWEN_XMLNODE *LC_Card_GetAppNode(const LC_CARD *card) {
+GWEN_XMLNODE *LC_Card_GetAppNode(const LC_CARD *card)
+{
   assert(card);
   return card->appNode;
 }
 
 
 
-LC_CLIENT_RESULT LC_Card_SelectCard(LC_CARD *card, const char *s) {
+LC_CLIENT_RESULT LC_Card_SelectCard(LC_CARD *card, const char *s)
+{
   assert(card);
   if (s==0)
     card->cardNode=0;
@@ -572,7 +752,8 @@ LC_CLIENT_RESULT LC_Card_SelectCard(LC_CARD *card, const char *s) {
 
 
 
-GWEN_XMLNODE *LC_Card_GetCardNode(const LC_CARD *card) {
+GWEN_XMLNODE *LC_Card_GetCardNode(const LC_CARD *card)
+{
   assert(card);
   return card->cardNode;
 }
@@ -582,7 +763,8 @@ GWEN_XMLNODE *LC_Card_GetCardNode(const LC_CARD *card) {
 
 GWEN_XMLNODE *LC_Card_FindFile(LC_CARD *card,
                                const char *type,
-                               const char *fname) {
+                               const char *fname)
+{
   GWEN_XMLNODE *n;
   GWEN_XMLNODE *currDF;
   int isSameLevel;
@@ -592,7 +774,7 @@ GWEN_XMLNODE *LC_Card_FindFile(LC_CARD *card,
     currDF=card->appNode;
 
   isSameLevel=1;
-  while(currDF) {
+  while (currDF) {
     n=GWEN_XMLNode_FindNode(currDF, GWEN_XMLNodeTypeTag, "files");
     if (n) {
       n=GWEN_XMLNode_FindFirstTag(n, type, "name", fname);
@@ -613,17 +795,56 @@ GWEN_XMLNODE *LC_Card_FindFile(LC_CARD *card,
   return 0;
 }
 
+GWEN_XMLNODE *LC_Card_FindFileById(LC_CARD *card,
+                                   const char *type,
+                                   const int sid)
+{
+  GWEN_XMLNODE *n;
+  GWEN_XMLNODE *currDF;
+  GWEN_XMLNODE *nn;
+  int isSameLevel;
+  int fid;
+  char sidstr[10];
+
+  currDF=card->dfNode;
+  if (!currDF)
+    currDF=card->appNode;
+
+  isSameLevel=1;
+
+  sprintf(sidstr, "%#x", sid);
+
+  while (currDF) {
+    n=GWEN_XMLNode_FindNode(currDF, GWEN_XMLNodeTypeTag, "files");
+    if (n) {
+      n=GWEN_XMLNode_FindFirstTag(n, type, "sid", sidstr);
+      if (n) {
+        if (isSameLevel) {
+          return n;
+        }
+        if (atoi(GWEN_XMLNode_GetProperty(n, "inAnyDF", "0"))!=0) {
+          DBG_DEBUG(LC_LOGDOMAIN, "Returning file from level above");
+          return n;
+        }
+      }
+    }
+    currDF=GWEN_XMLNode_GetParent(currDF);
+    isSameLevel=0;
+  }
+  DBG_DEBUG(LC_LOGDOMAIN, "%s \"%s\" not found", type, sidstr);
+  return 0;
+}
 
 
-LC_CLIENT_RESULT LC_Card_SelectMf(LC_CARD *card) {
+LC_CLIENT_RESULT LC_Card_SelectMf(LC_CARD *card)
+{
   GWEN_DB_NODE *dbReq;
   GWEN_DB_NODE *dbRsp;
   LC_CLIENT_RESULT res;
 
   dbReq=GWEN_DB_Group_new("request");
   dbRsp=GWEN_DB_Group_new("response");
-  res=LC_Card_ExecCommand(card, "SelectMF", dbReq, dbRsp,
-                          LC_Client_GetShortTimeout(card->client));
+  res=LC_Card_ExecCommand(card, "SelectMF", dbReq, dbRsp);
   GWEN_DB_Group_free(dbRsp);
   GWEN_DB_Group_free(dbReq);
   if (res!=LC_Client_ResultOk) {
@@ -637,7 +858,8 @@ LC_CLIENT_RESULT LC_Card_SelectMf(LC_CARD *card) {
 
 
 
-LC_CLIENT_RESULT LC_Card_SelectDf(LC_CARD *card, const char *fname) {
+LC_CLIENT_RESULT LC_Card_SelectDf(LC_CARD *card, const char *fname)
+{
   GWEN_XMLNODE *n;
   GWEN_DB_NODE *dbReq;
   GWEN_DB_NODE *dbRsp;
@@ -651,7 +873,7 @@ LC_CLIENT_RESULT LC_Card_SelectDf(LC_CARD *card, const char *fname) {
     return LC_Client_ResultCmdError;
   }
 
-  if (1!=sscanf(GWEN_XMLNode_GetProperty(n, "sid", "-1"), "%i", &fid)){
+  if (1!=sscanf(GWEN_XMLNode_GetProperty(n, "sid", "-1"), "%i", &fid)) {
     DBG_ERROR(LC_LOGDOMAIN, "Bad id for DF \"%s\"", fname);
     return LC_Client_ResultCmdError;
   }
@@ -690,8 +912,7 @@ LC_CLIENT_RESULT LC_Card_SelectDf(LC_CARD *card, const char *fname) {
   }
 
   dbRsp=GWEN_DB_Group_new("response");
-  res=LC_Card_ExecCommand(card, cmd, dbReq, dbRsp,
-                          LC_Client_GetShortTimeout(card->client));
+  res=LC_Card_ExecCommand(card, cmd, dbReq, dbRsp);
   GWEN_DB_Group_free(dbRsp);
   GWEN_DB_Group_free(dbReq);
   if (res!=LC_Client_ResultOk) {
@@ -706,14 +927,16 @@ LC_CLIENT_RESULT LC_Card_SelectDf(LC_CARD *card, const char *fname) {
 
 
 
-GWEN_XMLNODE *LC_Card_GetDfNode(const LC_CARD *card) {
+GWEN_XMLNODE *LC_Card_GetDfNode(const LC_CARD *card)
+{
   assert(card);
   return card->dfNode;
 }
 
 
 
-LC_CLIENT_RESULT LC_Card_SelectEf(LC_CARD *card, const char *fname) {
+LC_CLIENT_RESULT LC_Card_SelectEf(LC_CARD *card, const char *fname)
+{
   GWEN_XMLNODE *n;
   GWEN_DB_NODE *dbReq;
   GWEN_DB_NODE *dbRsp;
@@ -727,7 +950,7 @@ LC_CLIENT_RESULT LC_Card_SelectEf(LC_CARD *card, const char *fname) {
     return LC_Client_ResultCmdError;
   }
 
-  if (1!=sscanf(GWEN_XMLNode_GetProperty(n, "sid", "-1"), "%i", &fid)){
+  if (1!=sscanf(GWEN_XMLNode_GetProperty(n, "sid", "-1"), "%i", &fid)) {
     DBG_ERROR(LC_LOGDOMAIN, "Bad id for DF \"%s\"", fname);
     return LC_Client_ResultCmdError;
   }
@@ -766,8 +989,7 @@ LC_CLIENT_RESULT LC_Card_SelectEf(LC_CARD *card, const char *fname) {
   }
 
   dbRsp=GWEN_DB_Group_new("response");
-  res=LC_Card_ExecCommand(card, cmd, dbReq, dbRsp,
-                          LC_Client_GetShortTimeout(card->client));
+  res=LC_Card_ExecCommand(card, cmd, dbReq, dbRsp);
   GWEN_DB_Group_free(dbRsp);
   GWEN_DB_Group_free(dbReq);
   if (res!=LC_Client_ResultOk) {
@@ -779,16 +1001,111 @@ LC_CLIENT_RESULT LC_Card_SelectEf(LC_CARD *card, const char *fname) {
   return LC_Client_ResultOk;
 }
 
+LC_CLIENT_RESULT LC_Card_SelectEfById(LC_CARD *card, const int  sid)
+{
+  GWEN_XMLNODE *n;
+  GWEN_DB_NODE *dbReq;
+  GWEN_DB_NODE *dbRsp;
+  const char *cmd;
+  int fid;
+  LC_CLIENT_RESULT res;
+
+  n=LC_Card_FindFileById(card, "EF", sid);
+  if (!n) {
+    DBG_ERROR(LC_LOGDOMAIN, "EF \"%d\" not found", sid);
+    return LC_Client_ResultCmdError;
+  }
+
+  if (1!=sscanf(GWEN_XMLNode_GetProperty(n, "sid", "-1"), "%i", &fid)) {
+    DBG_ERROR(LC_LOGDOMAIN, "Bad id for DF \"%d\"", sid);
+    return LC_Client_ResultCmdError;
+  }
+
+  dbReq=GWEN_DB_Group_new("request");
+  if (fid==-1) {
+    GWEN_BUFFER *buf;
+    const char *lid;
+
+    buf=GWEN_Buffer_new(0, 64, 0, 1);
+    lid=GWEN_XMLNode_GetProperty(n, "lid", 0);
+    if (!lid) {
+      DBG_ERROR(LC_LOGDOMAIN, "No long id given in XML file");
+      GWEN_Buffer_free(buf);
+      GWEN_DB_Group_free(dbReq);
+      return LC_Client_ResultDataError;
+    }
+    if (GWEN_Text_FromHexBuffer(lid, buf)) {
+      DBG_ERROR(LC_LOGDOMAIN, "Bad long id given in XML file");
+      GWEN_Buffer_free(buf);
+      GWEN_DB_Group_free(dbReq);
+      return LC_Client_ResultDataError;
+    }
+
+    GWEN_DB_SetBinValue(dbReq, GWEN_DB_FLAGS_OVERWRITE_VARS,
+                        "fileId",
+                        GWEN_Buffer_GetStart(buf),
+                        GWEN_Buffer_GetUsedBytes(buf));
+    cmd="SelectEFL";
+
+  }
+  else {
+    GWEN_DB_SetIntValue(dbReq, GWEN_DB_FLAGS_OVERWRITE_VARS,
+                        "fileId", fid);
+    cmd="SelectEFS";
+  }
+
+  dbRsp=GWEN_DB_Group_new("response");
+  res=LC_Card_ExecCommand(card, cmd, dbReq, dbRsp);
+  GWEN_DB_Group_free(dbRsp);
+  GWEN_DB_Group_free(dbReq);
+  if (res!=LC_Client_ResultOk) {
+    DBG_INFO(LC_LOGDOMAIN, "here (%d)", res);
+    return res;
+  }
+  card->efNode=n;
+
+  return LC_Client_ResultOk;
+}
+
+LC_CLIENT_RESULT LC_Card_SelectEfByID(LC_CARD *card, const int fid)
+{
+  GWEN_XMLNODE *n;
+  GWEN_DB_NODE *dbReq;
+  GWEN_DB_NODE *dbRsp;
+  const char *cmd;
+  LC_CLIENT_RESULT res;
 
 
-GWEN_XMLNODE *LC_Card_GetEfNode(const LC_CARD *card) {
+  dbReq=GWEN_DB_Group_new("request");
+  GWEN_DB_SetIntValue(dbReq, GWEN_DB_FLAGS_OVERWRITE_VARS,
+                      "fileId", fid);
+  cmd="SelectEFS";
+
+
+  dbRsp=GWEN_DB_Group_new("response");
+  res=LC_Card_ExecCommand(card, cmd, dbReq, dbRsp);
+  GWEN_DB_Group_free(dbRsp);
+  GWEN_DB_Group_free(dbReq);
+  if (res!=LC_Client_ResultOk) {
+    DBG_INFO(LC_LOGDOMAIN, "here (%d)", res);
+    return res;
+  }
+
+  return LC_Client_ResultOk;
+}
+
+
+
+GWEN_XMLNODE *LC_Card_GetEfNode(const LC_CARD *card)
+{
   assert(card);
   return card->efNode;
 }
 
 
 
-LC_PININFO *LC_Card_GetPinInfoById(LC_CARD *card, uint32_t pid) {
+LC_PININFO *LC_Card_GetPinInfoById(LC_CARD *card, uint32_t pid)
+{
   GWEN_XMLNODE *n;
 
   n=card->efNode;
@@ -801,7 +1118,7 @@ LC_PININFO *LC_Card_GetPinInfoById(LC_CARD *card, uint32_t pid) {
     return 0;
   }
 
-  while(n) {
+  while (n) {
     GWEN_XMLNODE *nn;
 
     nn=GWEN_XMLNode_FindFirstTag(n, "pins", 0, 0);
@@ -809,7 +1126,7 @@ LC_PININFO *LC_Card_GetPinInfoById(LC_CARD *card, uint32_t pid) {
       GWEN_XMLNODE *nnn;
 
       nnn=GWEN_XMLNode_FindFirstTag(nn, "pin", 0, 0);
-      while(nnn) {
+      while (nnn) {
         const char *s;
 
         s=GWEN_XMLNode_GetProperty(nnn, "id", 0);
@@ -857,7 +1174,8 @@ LC_PININFO *LC_Card_GetPinInfoById(LC_CARD *card, uint32_t pid) {
 
 
 
-LC_PININFO *LC_Card_GetPinInfoByName(LC_CARD *card, const char *name) {
+LC_PININFO *LC_Card_GetPinInfoByName(LC_CARD *card, const char *name)
+{
   GWEN_XMLNODE *n;
 
   assert(card);
@@ -877,7 +1195,7 @@ LC_PININFO *LC_Card_GetPinInfoByName(LC_CARD *card, const char *name) {
     return 0;
   }
 
-  while(n) {
+  while (n) {
     GWEN_XMLNODE *nn;
 
     DBG_DEBUG(LC_LOGDOMAIN, "Searching in \"%s\" (%s)",
@@ -889,7 +1207,7 @@ LC_PININFO *LC_Card_GetPinInfoByName(LC_CARD *card, const char *name) {
       GWEN_XMLNODE *nnn;
 
       nnn=GWEN_XMLNode_FindFirstTag(nn, "pin", 0, 0);
-      while(nnn) {
+      while (nnn) {
         const char *s;
         int i;
 
@@ -938,7 +1256,8 @@ LC_PININFO *LC_Card_GetPinInfoByName(LC_CARD *card, const char *name) {
 LC_CLIENT_RESULT LC_Card_ParseData(LC_CARD *card,
                                    const char *format,
                                    GWEN_BUFFER *buf,
-                                   GWEN_DB_NODE *dbData) {
+                                   GWEN_DB_NODE *dbData)
+{
   GWEN_XMLNODE *dataNode;
   GWEN_MSGENGINE *e;
 
@@ -968,7 +1287,7 @@ LC_CLIENT_RESULT LC_Card_ParseData(LC_CARD *card,
                                   dataNode,
                                   buf,
                                   dbData,
-                                  GWEN_MSGENGINE_READ_FLAGS_DEFAULT)){
+                                  GWEN_MSGENGINE_READ_FLAGS_DEFAULT)) {
     DBG_ERROR(LC_LOGDOMAIN, "Error parsing data in format \"%s\"", format);
     return LC_Client_ResultDataError;
   }
@@ -981,7 +1300,8 @@ LC_CLIENT_RESULT LC_Card_ParseData(LC_CARD *card,
 LC_CLIENT_RESULT LC_Card_CreateData(LC_CARD *card,
                                     const char *format,
                                     GWEN_BUFFER *buf,
-                                    GWEN_DB_NODE *dbData){
+                                    GWEN_DB_NODE *dbData)
+{
   GWEN_XMLNODE *dataNode;
   GWEN_MSGENGINE *e;
 
@@ -1020,7 +1340,8 @@ LC_CLIENT_RESULT LC_Card_CreateData(LC_CARD *card,
 LC_CLIENT_RESULT LC_Card_ParseRecord(LC_CARD *card,
                                      int recNum,
                                      GWEN_BUFFER *buf,
-                                     GWEN_DB_NODE *dbRecord){
+                                     GWEN_DB_NODE *dbRecord)
+{
   GWEN_XMLNODE *recordNode;
   GWEN_MSGENGINE *e;
 
@@ -1033,7 +1354,7 @@ LC_CLIENT_RESULT LC_Card_ParseRecord(LC_CARD *card,
     return LC_Client_ResultNoData;
   }
   recordNode=GWEN_XMLNode_FindFirstTag(card->efNode, "record", 0, 0);
-  while(recordNode) {
+  while (recordNode) {
     int lrecNum;
 
     if (1==sscanf(GWEN_XMLNode_GetProperty(recordNode,
@@ -1045,7 +1366,7 @@ LC_CLIENT_RESULT LC_Card_ParseRecord(LC_CARD *card,
     recordNode=GWEN_XMLNode_FindNextTag(recordNode, "record", 0, 0);
   } /* while */
   if (!recordNode)
-    recordNode=GWEN_XMLNode_FindFirstTag(card->efNode,"record", 0, 0);
+    recordNode=GWEN_XMLNode_FindFirstTag(card->efNode, "record", 0, 0);
 
   if (recordNode) {
     /* node found, parse data */
@@ -1054,7 +1375,7 @@ LC_CLIENT_RESULT LC_Card_ParseRecord(LC_CARD *card,
                                     recordNode,
                                     buf,
                                     dbRecord,
-                                    GWEN_MSGENGINE_READ_FLAGS_DEFAULT)){
+                                    GWEN_MSGENGINE_READ_FLAGS_DEFAULT)) {
       DBG_ERROR(LC_LOGDOMAIN, "Error parsing response");
       return LC_Client_ResultDataError;
     }
@@ -1071,7 +1392,8 @@ LC_CLIENT_RESULT LC_Card_ParseRecord(LC_CARD *card,
 LC_CLIENT_RESULT LC_Card_CreateRecord(LC_CARD *card,
                                       int recNum,
                                       GWEN_BUFFER *buf,
-                                      GWEN_DB_NODE *dbRecord){
+                                      GWEN_DB_NODE *dbRecord)
+{
   GWEN_XMLNODE *recordNode;
   GWEN_MSGENGINE *e;
 
@@ -1080,7 +1402,7 @@ LC_CLIENT_RESULT LC_Card_CreateRecord(LC_CARD *card,
   e=LC_Client_GetMsgEngine(card->client);
   assert(e);
   recordNode=GWEN_XMLNode_FindFirstTag(card->efNode, "record", 0, 0);
-  while(recordNode) {
+  while (recordNode) {
     int lrecNum;
 
     if (1==sscanf(GWEN_XMLNode_GetProperty(recordNode,
@@ -1092,7 +1414,7 @@ LC_CLIENT_RESULT LC_Card_CreateRecord(LC_CARD *card,
     recordNode=GWEN_XMLNode_FindNextTag(recordNode, "record", 0, 0);
   } /* while */
   if (!recordNode)
-    recordNode=GWEN_XMLNode_FindFirstTag(card->efNode,"record", 0, 0);
+    recordNode=GWEN_XMLNode_FindFirstTag(card->efNode, "record", 0, 0);
 
   if (recordNode) {
     /* node found, parse data */
@@ -1117,7 +1439,8 @@ LC_CLIENT_RESULT LC_Card_CreateRecord(LC_CARD *card,
 LC_CLIENT_RESULT LC_Card_GetPinStatus(LC_CARD *card,
                                       unsigned int pid,
                                       int *maxErrors,
-                                      int *currentErrors) {
+                                      int *currentErrors)
+{
   assert(card);
   if (card->getPinStatusFn) {
     return card->getPinStatusFn(card, pid, maxErrors, currentErrors);
@@ -1135,7 +1458,8 @@ LC_CLIENT_RESULT LC_Card_GetInitialPin(LC_CARD *card,
                                        int id,
                                        unsigned char *buffer,
                                        unsigned int maxLen,
-                                       unsigned int *pinLength) {
+                                       unsigned int *pinLength)
+{
   assert(card);
   if (card->getInitialPinFn) {
     return card->getInitialPinFn(card, id, buffer, maxLen, pinLength);
@@ -1149,42 +1473,48 @@ LC_CLIENT_RESULT LC_Card_GetInitialPin(LC_CARD *card,
 
 
 
-LC_CARD_OPEN_FN LC_Card_GetOpenFn(const LC_CARD *card){
+LC_CARD_OPEN_FN LC_Card_GetOpenFn(const LC_CARD *card)
+{
   assert(card);
   return card->openFn;
 }
 
 
 
-void LC_Card_SetOpenFn(LC_CARD *card, LC_CARD_OPEN_FN fn){
+void LC_Card_SetOpenFn(LC_CARD *card, LC_CARD_OPEN_FN fn)
+{
   assert(card);
   card->openFn=fn;
 }
 
 
 
-LC_CARD_CLOSE_FN LC_Card_GetCloseFn(const LC_CARD *card){
+LC_CARD_CLOSE_FN LC_Card_GetCloseFn(const LC_CARD *card)
+{
   assert(card);
   return card->closeFn;
 }
 
 
 
-void LC_Card_SetCloseFn(LC_CARD *card, LC_CARD_CLOSE_FN fn){
+void LC_Card_SetCloseFn(LC_CARD *card, LC_CARD_CLOSE_FN fn)
+{
   assert(card);
   card->closeFn=fn;
 }
 
 
 
-void LC_Card_SetGetPinStatusFn(LC_CARD *card, LC_CARD_GETPINSTATUS_FN fn) {
+void LC_Card_SetGetPinStatusFn(LC_CARD *card, LC_CARD_GETPINSTATUS_FN fn)
+{
   assert(card);
   card->getPinStatusFn=fn;
 }
 
 
 
-void LC_Card_SetGetInitialPinFn(LC_CARD *card, LC_CARD_GETINITIALPIN_FN fn){
+void LC_Card_SetGetInitialPinFn(LC_CARD *card, LC_CARD_GETINITIALPIN_FN fn)
+{
   assert(card);
   card->getInitialPinFn=fn;
 }
@@ -1194,10 +1524,11 @@ void LC_Card_SetGetInitialPinFn(LC_CARD *card, LC_CARD_GETINITIALPIN_FN fn){
 void LC_Card_CreateResultString(const LC_CARD *card,
                                 const char *lastCommand,
                                 LC_CLIENT_RESULT res,
-                                GWEN_BUFFER *buf) {
+                                GWEN_BUFFER *buf)
+{
   const char *s;
 
-  switch(res) {
+  switch (res) {
   case LC_Client_ResultOk:
     s="Ok.";
     break;
@@ -1268,14 +1599,18 @@ void LC_Card_CreateResultString(const LC_CARD *card,
 
 
 LC_CLIENT_RESULT LC_Card_ReadBinary(LC_CARD *card,
-				    int offset,
-				    int size,
-				    GWEN_BUFFER *buf){
+                                    int offset,
+                                    int size,
+                                    GWEN_BUFFER *buf)
+{
   int t;
   int bytesRead=0;
   LC_CLIENT_RESULT res;
 
-  while(size>0) {
+  while (size>0) {
+    int sw1;
+    int sw2;
+
     if (size>252)
       t=252;
     else
@@ -1291,6 +1626,14 @@ LC_CLIENT_RESULT LC_Card_ReadBinary(LC_CARD *card,
     size-=t;
     offset+=t;
     bytesRead+=t;
+
+    /* check for EOF */
+    sw1=LC_Card_GetLastSW1(card);
+    sw2=LC_Card_GetLastSW2(card);
+    if (sw1==0x62 && sw2==0x82) {
+      DBG_DEBUG(LC_LOGDOMAIN, "EOF met after %d bytes (asked for %d bytes more)", bytesRead, size);
+      break;
+    }
   } /* while still data to read */
 
   return LC_Client_ResultOk;
