@@ -1,9 +1,6 @@
 /***************************************************************************
- $RCSfile$
-                             -------------------
-    cvs         : $Id$
     begin       : Mon Mar 01 2004
-    copyright   : (C) 2004 by Martin Preuss
+    copyright   : (C) 2004-2010 by Martin Preuss
     email       : martin@libchipcard.de
 
  ***************************************************************************
@@ -14,7 +11,7 @@
 #ifndef CHIPCARD_CARD_DDVCARD_H
 #define CHIPCARD_CARD_DDVCARD_H
 
-#include <chipcard/client/card.h>
+#include <chipcard/card.h>
 
 #ifdef __cplusplus
 extern "C" {
@@ -141,17 +138,17 @@ CHIPCARD_API GWEN_BUFFER *LC_DDVCard_GetCardDataAsBuffer(const LC_CARD *card);
  * Read institution data (including bank code, server address, user id etc).
  */
 CHIPCARD_API
-  LC_CLIENT_RESULT LC_DDVCard_ReadInstituteData(LC_CARD *card,
-                                                int idx,
-                                                GWEN_DB_NODE *dbData);
+LC_CLIENT_RESULT LC_DDVCard_ReadInstituteData(LC_CARD *card,
+                                              int idx,
+                                              GWEN_DB_NODE *dbData);
 
 /**
  * Write institution data (including bank code, server address, user id etc).
  */
-CHIPCARD_API 
-  LC_CLIENT_RESULT LC_DDVCard_WriteInstituteData(LC_CARD *card,
-                                                 int idx,
-                                                 GWEN_DB_NODE *dbData);
+CHIPCARD_API
+LC_CLIENT_RESULT LC_DDVCard_WriteInstituteData(LC_CARD *card,
+                                               int idx,
+                                               GWEN_DB_NODE *dbData);
 /*@}*/
 
 /** @name Key Information
