@@ -1,9 +1,6 @@
 /***************************************************************************
- $RCSfile$
-                             -------------------
-    cvs         : $Id$
     begin       : Mon Mar 01 2004
-    copyright   : (C) 2004 by Martin Preuss
+    copyright   : (C) 2004-2010 by Martin Preuss
     email       : martin@libchipcard.de
 
  ***************************************************************************
@@ -14,8 +11,8 @@
 #ifndef CHIPCARD_CARD_PROCESSORCARD_P_H
 #define CHIPCARD_CARD_PROCESSORCARD_P_H
 
-#include <chipcard/client/card_imp.h>
-#include <chipcard/client/cards/processorcard.h>
+#include <chipcard/card_imp.h>
+#include <chipcard/cards/processorcard.h>
 
 
 typedef struct LC_PROCESSORCARD LC_PROCESSORCARD;
